<?php

class Backend_Customers_Controller extends Backend_Controller {
	
	public $restful = true;

    // Validation rules
    private $rules = array(
        'type' => 'required',
        'type_inscription' => 'required',
        'first_name' => 'required',
        'last_name' => 'required'
    );

	public function __construct()
    {
    	parent::__construct();
        
        $this->filter('before', 'permission_check:43');
        
        $this->array_types = array(
        	'op_interne' => 'OP Interne',
        	'invite' => 'Invité',
        	'labo' => 'Labo',
        	'intervenant' => 'Intervenant',
        	'titulaire' => 'Titulaire',
        	'accompagnant' => 'Accompagnant',
        	'enfant' => 'Enfant'
        );
        $this->array_types_inscriptions = array(
        	'Wellpharma' => 'Wellpharma',
        	'Séminnov' => 'Séminnov',
        	'Wellpharma + Séminnov' => 'Wellpharma + Séminnov',
        );
        
        // Années
        $this->years = array();
        for($i=(date('Y')+10);$i>=(date('Y')-30);$i--){
	        $this->years[$i] = $i;
        }
    }
    
    public function get_ajaxcustomers(){
    	$input = Input::all();
    	
	    // Renvoie les infos en ajax au Datatable pour les customers (table trop lourde pour tout lister d'un coup)
	    $aColumns = array('customers.id', 'type', 'customers.last_name', 'customers.first_name', 'customers.code_cip','pharma_nom','pharma_cp','type_inscription' );
	    $sIndexColumn = "customers.id";
	    $sLimit = "";
	    $sTable = 'customers';
	    
	    if ( isset( $input['iDisplayStart'] ) && $input['iDisplayLength'] != '-1' )
		{
			$sLimit = "LIMIT ".intval( $input['iDisplayStart'] ).", ".intval( $input['iDisplayLength'] );
		}
		
		$sOrder = "";
		if ( isset( $input['iSortCol_0'] ) )
		{
			$sOrder = "ORDER BY  ";
			for ( $i=0 ; $i<intval( $input['iSortingCols'] ) ; $i++ )
			{
				if ( $input[ 'bSortable_'.intval($input['iSortCol_'.$i]) ] == "true" )
				{
					$sOrder .= "".$aColumns[ intval( $input['iSortCol_'.$i] ) ]." ".
						($input['sSortDir_'.$i]==='asc' ? 'asc' : 'desc') .", ";
				}
			}
			
			$sOrder = substr_replace( $sOrder, "", -2 );
			if ( $sOrder == "ORDER BY" )
			{
				$sOrder = "";
			}
		}
		
		if($input['force']=='noscan'){
			$sWhere = "WHERE customers.id>=1 AND customers.id NOT IN (SELECT customer_id FROM customer_scans) ";
		}
		elseif($input['force']=='partialscan'){
			$sWhere = "WHERE customers.id>=1 AND customers.id IN (SELECT customer_id FROM customer_scans) ";
		}
		else{
			$sWhere = "WHERE customers.id>=1 ";
		}
		$nicocount = $sWhere;
		
		if ( isset($input['sSearch']) && $input['sSearch'] != "" )
		{
			$sWhere = $sWhere."AND (";
			for ( $i=0 ; $i<count($aColumns) ; $i++ )
			{
				$sWhere .= "".$aColumns[$i]." LIKE '%".$input['sSearch']."%' OR ";
			}
			$sWhere = substr_replace( $sWhere, "", -3 );
			$sWhere .= ')';
		}
		
		/* Individual column filtering */
		for ( $i=0 ; $i<count($aColumns) ; $i++ )
		{
			if ( isset($input['bSearchable_'.$i]) && $input['bSearchable_'.$i] == "true" && $input['sSearch_'.$i] != '' )
			{
				if ( $sWhere == "" )
				{
					$sWhere = "WHERE ";
				}
				else
				{
					$sWhere .= " AND ";
				}
				$sWhere .= "".$aColumns[$i]." LIKE '%".$input['sSearch_'.$i]."%' ";
			}
		}
		
		$sQuery = "SELECT SQL_CALC_FOUND_ROWS customers.id,".str_replace(" , ", " ", implode(", ", $aColumns))." FROM $sTable $sWhere $sOrder $sLimit";
		//echo $sQuery;die();
		$rResult = DB::query($sQuery);
		
		/* Data set length after filtering */
		$sQuery = "SELECT FOUND_ROWS() total";
		$aResultFilterTotal = DB::query($sQuery);
		$iFilteredTotal = $aResultFilterTotal[0]->total;
		/* Total data set length */
		$sQuery = "SELECT COUNT(".$sIndexColumn.") total FROM $sTable $nicocount";
		$aResultTotal = DB::query($sQuery);
		$iTotal = $aResultTotal[0]->total;;
		
		$output = array(
			"sEcho" => intval($input['sEcho']),
			"iTotalRecords" => $iTotal,
			"iTotalDisplayRecords" => $iFilteredTotal,
			"aaData" => array()
		);
		
		foreach ( $rResult as $aRow )
		{
			$cur_customer = Customer::find($aRow->id);
			
			$row = array();
			
			// Ajout du pays en premier
			//$row[] = $cur_company->country->fr;
			
			for ( $i=0 ; $i<count($aColumns) ; $i++ )
			{
				if ( $aColumns[$i] == "version" )
				{
					/* Special output formatting for 'version' column */
					$row[] = ($aRow->$aColumns[$i]=="0") ? '-' : $aRow->$aColumns[$i];
				}
				else if ( $aColumns[$i] != ' ' )
				{
					/* General output */
					if($aColumns[$i]=="customers.created_at" || $aColumns[$i]=="customers.updated_at"){
						$column = str_replace(array('companies.','customers.'),'',$aColumns[$i]);
						$row[] = substr($aRow->$column,0,10);
					}
					elseif($aColumns[$i]=="type"){
						$row[] = ucfirst($aRow->$aColumns[$i]);
					}
					else{
						$column = str_replace(array('companies.','customers.'),'',$aColumns[$i]);
						$row[] = $aRow->$column;
					}
				}
			}
			
			// Champs perso - modification etc
			$row[] = Form::open(URL::to_route('delete_customer', $cur_customer->id), 'DELETE', array('id' => 'form_'.$cur_customer->id, 'style' => 'margin:0;')).'
						<div class="btn-group" style="float:right">
							<a class="btn btn-small" href="'.URL::to_route('edit_customer', array($cur_customer->id)).'"><i class="icon-pencil"></i> Modifier</a>
							<a class="btn btn-small btn-danger" href="javascript:delete_entry('.$cur_customer->id.');"><i class="icon-remove icon-trash"></i></a>
						</div>
						'.Form::close();
			
			$output['aaData'][] = $row;
		}
		
		echo json_encode( $output );
    }

    public function get_index()
    {
        $data['page_title'] = 'Tous les inscrits';
        return View::make('backend::customers.index', $data);
    }

    public function get_create()
    {
    	Asset::container('header')->add('uploadifive-css', 'css/uploadifive.css');
        Asset::container('footer')->add('uploadifive-js', 'js/jquery.uploadifive.min.js');
        Asset::container('header')->add('product-js', 'js/product.js');

    	$data['types'] = $this->array_types;
    	$data['types_inscriptions'] = $this->array_types_inscriptions;
        $data['page_title'] = 'Ajouter un nouvel inscrit';
        return View::make('backend::customers.create', $data);
    }

    public function post_store()
    {
        $input = Input::all();
        $v = Validator::make($input, $this->rules);

        if($v->fails())
        {
            return Redirect::to_route('new_customer')
                ->with('errors', $v->errors)
                ->with_input();
        }
        else
        {
        	$customer = Customer::create(array(
        		'user_id' => Auth::user()->id,
            	'first_name' => $input['first_name'],
            	'last_name' => $input['last_name'],
            	'type' => $input['type'],
            	'type_inscription' => $input['type_inscription'],
                'code_cip' => $input['code_cip'],
                'pharma_nom' => $input['pharma_nom'],
                'pharma_cp' => $input['pharma_cp']
			));
			
			// Files
            if(isset($input['files']))
            {
                $j = 1;
                foreach($input['files'] as $f)
                {
                    $cfile = FileU::find($f);
                    $cfile->customer_id = $customer->id;
                    $cfile->position = $j++;
                    $cfile->save();
                }
            }
            
            return Redirect::to_route('customers')
                ->with('type', 'success')
                ->with('message', 'Inscrit ajouté avec succès.');
        }
    }
    
    public function generer_qrcode($id,$size=500){
    	if(!$customer = Customer::find($id)){return Redirect::to_route('customers');}
	    $qr = new QR();
	    $qr->text('spiceup'.substr(md5('sPiCe@'.$id.'uP'),0,10).$id);
	    return $qr->draw($size);
    }
    
    public function get_edit($id)
    {
        if(!$customer = Customer::find($id)){return Redirect::to_route('customers');}
		
		Asset::container('header')->add('uploadifive-css', 'css/uploadifive.css');
        Asset::container('footer')->add('uploadifive-js', 'js/jquery.uploadifive.min.js');
        Asset::container('header')->add('product-js', 'js/product.js');
        
    	$data['types'] = $this->array_types;
    	$data['types_inscriptions'] = $this->array_types_inscriptions;
        $data['customer'] = $customer;
        $data['page_title'] = 'Modifier un inscrit';
        $data['qr_code'] = $this->generer_qrcode($id,250);
        return View::make('backend::customers.edit', $data);
    }

    public function put_update($id)
    {
        if(!$customer = Customer::find($id)){return Redirect::to_route('customers');}
        
    	$input = Input::all();
        $v = Validator::make($input, $this->rules);

        if($v->fails())
        {
            return Redirect::to_route('edit_customer', $id)
                ->with('errors', $v->errors)
                ->with_input();
        }
        else
        {
        	$array_customer = array(
        		'user_id' => Auth::user()->id,
            	'first_name' => $input['first_name'],
            	'last_name' => $input['last_name'],
            	'type' => $input['type'],
            	'type_inscription' => $input['type_inscription'],
                'code_cip' => $input['code_cip'],
                'pharma_nom' => $input['pharma_nom'],
                'pharma_cp' => $input['pharma_cp']
			);
            
            $customer->fill($array_customer);
            $customer->save();
            
             // Files
            DB::table('files')->where('customer_id', '=', $customer->id)->update(array('customer_id' => 0));
            if(isset($input['files']))
            {
                $j = 1;
                foreach($input['files'] as $f)
                {
                	$cfile = FileU::find($f);
                	if($cfile){
	                    $cfile->customer_id = $customer->id;
	                    $cfile->position = $j++;
	                    $cfile->save();
                    }
                }
            }
            
            // Suppression des fichiers non attribués
	        $files = FileU::where('customer_id','=','0')->get();
	        foreach($files as $f){
	        	@unlink(path('public').'uploads/files/customers/'.$f->name);
	        	$f->delete();
	        }

            return Redirect::to_route('customers')
                ->with('type', 'info')
                ->with('message','Inscrit modifié avec succès.');
        }
    }

    public function delete_destroy($id)
    {
    	if(!$customer = Customer::find($id)){return Redirect::to_route('customers');}
    	
    	// Suppression des fichiers
        $files = FileU::where('customer_id','=',$id)->get();
        foreach($files as $f){
        	@unlink(path('public').'uploads/files/customers/'.$f->name);
        	$f->delete();
        }
        
        $customer->delete();

        return Redirect::to_route('customers')
            ->with('type', 'error')
            ->with('message', 'Inscrit modifié avec succès.');
    }
    
    public function post_file()
    {
    	// Fonction appelée par le script ajax du _form de création & dans le product.js
        // Quick fix for profiler showing on local env 
        Config::set('application.profiler', false);

        // File upload logic
        $input = Input::all();
        $file = Input::file('Filedata');


        $upload = array('upload_dir' => path('public').'uploads/files/customers/');
        $type = 'FileU';
        $filename = uniqid().'-'.GeneralFunctions::cleanstring($file['name']).'.'.File::extension($file['name']);

        if(Input::upload('Filedata', $upload['upload_dir'], $filename))
        {
            $newfile = $type::create(array(
                'name' => $filename
            ));

            $url_to_asset = URL::to_asset('uploads/files/customers/'.$filename);

            return $newfile->id.'|'.$url_to_asset;
        }
        else
        {
            return false;
        }
    }
    
    public function get_qrcodepdf($id){
    	if(!$customer = Customer::find($id)){return Redirect::to_route('customers');}
    	
        /***** Génére le pdf ******/
        $path='/home/spiceupfrb/www_events/bundles/html2pdf/libs/';
        //'/bundles/html2pdf/libs/';
        require_once($path."html2pdf.class.php");
		
		$data['qr_code'] = $this->generer_qrcode($id,500);
		
        $html2pdf = new HTML2PDF('P','A4','fr',true,'UTF-8',array(0,0,0,0));
        $html2pdf->WriteHTML(View::make('backend::customers.pdf',$data));

        return Response::make($html2pdf->Output(), 200, array('Content-type' => 'application/pdf'));
    }
}