<?php

class Backend_Scans_Controller extends Backend_Controller {
	
	public $restful = true;

    // Validation rules
    private $rules = array(
        'customer_id' => 'required',
        'date' => 'after:2014-01-01'
    );

	public function __construct()
    {
    	parent::__construct();
        
        $this->filter('before', 'permission_check:44');
        
        $this->customers = array();
        $customers = Customer::order_by('last_name','ASC')->get();
        foreach($customers as $c){
        	$nom = trim($c->last_name.' '.$c->first_name);
        	$this->customers[$c->id] = trim(($nom != '' ? $nom.', ' : '').$c->pharma_nom.($c->pharma_cp != '' ? ' '.$c->pharma_cp : ''));
        }
    }
    
    public function get_exportcsv(){
    	$scans = CustomerScan::order_by('id','ASC')->get();
        header('Content-Type: text/csv; charset=utf-8');
        header('Content-Disposition: attachment; filename=data.csv');
        $output = fopen('php://output', 'w');
        fputcsv($output, array('#', 'device', 'inscrit', 'date'));
        
        foreach($scans as $row){
        	$nom = trim($row->customer->last_name.' '.$row->customer->first_name);
        	$inscrit = trim(($nom != '' ? $nom.', ' : '').$row->customer->pharma_nom.($row->customer->pharma_cp != '' ? ' '.$row->customer->pharma_cp : ''));
        	
        	$array_row = array(
        		$row->id,
        		$row->device,
        		$inscrit,
        		$row->date
        	);
        	fputcsv($output, $array_row); 
        }
    }
    
    public function get_ajaxscans(){
    	$input = Input::all();
    	
	    // Renvoie les infos en ajax au Datatable
	    $aColumns = array('customer_scans.id', 'customer_scans.device', 'customer_scans.customer_id', 'customer_scans.date' );
	    $sIndexColumn = "customer_scans.id";
	    $sLimit = "";
	    $sTable = 'customer_scans';
	    
	    if ( isset( $input['iDisplayStart'] ) && $input['iDisplayLength'] != '-1' )
		{
			$sLimit = "LIMIT ".intval( $input['iDisplayStart'] ).", ".intval( $input['iDisplayLength'] );
		}
		
		$sOrder = "";
		if ( isset( $input['iSortCol_0'] ) )
		{
			$sOrder = "ORDER BY  ";
			for ( $i=0 ; $i<intval( $input['iSortingCols'] ) ; $i++ )
			{
				if ( $input[ 'bSortable_'.intval($input['iSortCol_'.$i]) ] == "true" )
				{
					$sOrder .= "".$aColumns[ intval( $input['iSortCol_'.$i] ) ]." ".
						($input['sSortDir_'.$i]==='asc' ? 'asc' : 'desc') .", ";
				}
			}
			
			$sOrder = substr_replace( $sOrder, "", -2 );
			if ( $sOrder == "ORDER BY" )
			{
				$sOrder = "";
			}
		}
		
		$sWhere = "WHERE customer_scans.id>=1 ";
		if ( isset($input['sSearch']) && $input['sSearch'] != "" )
		{
			$sWhere = $sWhere."AND (";
			for ( $i=0 ; $i<count($aColumns) ; $i++ )
			{
				$sWhere .= "".$aColumns[$i]." LIKE '%".$input['sSearch']."%' OR ";
			}
			$sWhere = substr_replace( $sWhere, "", -3 );
			$sWhere .= ')';
		}
		
		/* Individual column filtering */
		for ( $i=0 ; $i<count($aColumns) ; $i++ )
		{
			if ( isset($input['bSearchable_'.$i]) && $input['bSearchable_'.$i] == "true" && $input['sSearch_'.$i] != '' )
			{
				if ( $sWhere == "" )
				{
					$sWhere = "WHERE ";
				}
				else
				{
					$sWhere .= " AND ";
				}
				$sWhere .= "".$aColumns[$i]." LIKE '%".$input['sSearch_'.$i]."%' ";
			}
		}
		
		$sQuery = "SELECT SQL_CALC_FOUND_ROWS customer_scans.id,".str_replace(" , ", " ", implode(", ", $aColumns))." FROM $sTable $sWhere $sOrder $sLimit";
		//echo $sQuery;die();
		$rResult = DB::query($sQuery);
		
		/* Data set length after filtering */
		$sQuery = "SELECT FOUND_ROWS() total";
		$aResultFilterTotal = DB::query($sQuery);
		$iFilteredTotal = $aResultFilterTotal[0]->total;
		/* Total data set length */
		$sQuery = "SELECT COUNT(".$sIndexColumn.") total FROM $sTable WHERE customer_scans.id >=1 ";
		$aResultTotal = DB::query($sQuery);
		$iTotal = $aResultTotal[0]->total;;
		
		$output = array(
			"sEcho" => intval($input['sEcho']),
			"iTotalRecords" => $iTotal,
			"iTotalDisplayRecords" => $iFilteredTotal,
			"aaData" => array()
		);
		
		foreach ( $rResult as $aRow )
		{
			$cur_customer = Customer::find($aRow->customer_id);
			$row = array();
			
			for ( $i=0 ; $i<count($aColumns) ; $i++ )
			{
				if ( $aColumns[$i] == "version" )
				{
					/* Special output formatting for 'version' column */
					$row[] = ($aRow->$aColumns[$i]=="0") ? '-' : $aRow->$aColumns[$i];
				}
				else if ( $aColumns[$i] != ' ' )
				{
					/* General output */
					if($aColumns[$i]=="customer_scans.updated_at" || $aColumns[$i]=="customer_scans.created_at"){
						$column = str_replace(array('companies.','customer_scans.'),'',$aColumns[$i]);
						$row[] = substr($aRow->$column,0,10);
					}
					elseif($aColumns[$i]=="customer_scans.customer_id"){
						$nom = trim($cur_customer->last_name.' '.$cur_customer->first_name);
						$row[] = '<a href="'.URL::to_route('edit_customer', array($cur_customer->id)).'">'.trim(($nom != '' ? $nom.', ' : '').$cur_customer->pharma_nom.($cur_customer->pharma_cp != '' ? ' '.$cur_customer->pharma_cp : '')).'</a>';
					}
					else{
						$column = str_replace(array('companies.','customer_scans.'),'',$aColumns[$i]);
						$row[] = $aRow->$column;
					}
				}
			}
			
			// Champs perso - modification etc
			$row[] = Form::open(URL::to_route('delete_scan', $aRow->id), 'DELETE', array('id' => 'form_'.$aRow->id, 'style' => 'margin:0;')).'
						<div class="btn-group" style="float:right">
							<a class="btn btn-small" href="'.URL::to_route('edit_scan', array($aRow->id)).'"><i class="icon-pencil"></i> Modifier</a>
							<a class="btn btn-small btn-danger" href="javascript:delete_entry('.$aRow->id.');"><i class="icon-remove icon-trash"></i></a>
						</div>
						'.Form::close();
			
			$output['aaData'][] = $row;
		}
		
		echo json_encode( $output );
    }

    public function get_index()
    {
        $data['page_title'] = 'Tous les scans';
        $data['count'] =  CustomerScan::count();
        return View::make('backend::scans.index', $data);
    }

    public function get_create()
    {
    	Asset::container('header')->add('uploadifive-css', 'css/uploadifive.css');
        Asset::container('footer')->add('uploadifive-js', 'js/jquery.uploadifive.min.js');
        Asset::container('header')->add('product-js', 'js/product.js');

    	$data['customers'] = $this->customers;
        $data['page_title'] = 'Ajouter un nouveau scan';
        return View::make('backend::scans.create', $data);
    }

    public function post_store()
    {
        $input = Input::all();
        $v = Validator::make($input, $this->rules);

        if($v->fails())
        {
            return Redirect::to_route('new_scan')
                ->with('errors', $v->errors)
                ->with_input();
        }
        else
        {
        	$scan = CustomerScan::create(array(
            	'customer_id' => $input['customer_id'],
            	'date' => $input['date']
			));
			
            return Redirect::to_route('scans')
                ->with('type', 'success')
                ->with('message', 'Inscrit ajouté avec succès.');
        }
    }
    
    public function get_edit($id)
    {
        if(!$scan = CustomerScan::find($id)){return Redirect::to_route('scans');}
        
    	$data['customers'] = $this->customers;
    	$data['types_inscriptions'] = $this->array_types_inscriptions;
        $data['customer_scan'] = $scan;
        $data['page_title'] = 'Modifier un scan';
        
        return View::make('backend::scans.edit', $data);
    }

    public function put_update($id)
    {
        if(!$scan = CustomerScan::find($id)){return Redirect::to_route('scans');}
        
    	$input = Input::all();
        $v = Validator::make($input, $this->rules);

        if($v->fails())
        {
            return Redirect::to_route('edit_customer', $id)
                ->with('errors', $v->errors)
                ->with_input();
        }
        else
        {
        	$array_scan = array(
        		'customer_id' => $input['customer_id'],
            	'date' => $input['date']
			);
            
            $scan->fill($array_scan);
            $scan->save();

            return Redirect::to_route('scans')
                ->with('type', 'info')
                ->with('message','Scan modifié avec succès.');
        }
    }

    public function delete_destroy($id)
    {
    	if(!$scan = CustomerScan::find($id)){return Redirect::to_route('scans');}
        $scan->delete();

        return Redirect::to_route('scans')
            ->with('type', 'error')
            ->with('message', 'Scan modifié avec succès.');
    }
}