<?php

class Backend_Users_Controller extends Backend_Controller {
	
	public $restful = true;

    // Validation rules
    private $rules = array(
        'email' => 'required|email|unique:users',
        'first_name' => 'required|max:150|min:3',
        'password' => 'required|min:5'
    );

	public function __construct()
    {
    	parent::__construct();
    	
    	$this->filter('before', 'permission_check:1');
    }

    public function get_index()
    {
    	//$data['users'] = User::with(array('last_login'))->get();
    	$data['users'] = User::all();
    	$data['page_title'] = 'Administrateurs';
        return View::make('backend::users.index', $data);
    }

    public function get_create()
    {
        $data['roles'] = Role::order_by('name','ASC')->get();
        $data['page_title'] = 'Création d\'un nouveau compte administrateur';
        return View::make('backend::users.create', $data);
    }

    public function post_store()
    {
        $input = Input::all();
        $v = Validator::make($input, $this->rules);

        if($v->fails())
        {
            return Redirect::to_route('new_user')
                ->with('errors', $v->errors)
                ->with_input();
        }
        else
        {
            $user = User::create(array(
                'parent_id' => Auth::user()->id,
                'email' => $input['email'],
                'phone' => $input['phone'],
                'first_name' => $input['first_name'],
                'last_name' => $input['last_name'],
                'password' => Hash::make($input['password'])
            ));
            if(isset($input['roles'])){$user->roles()->sync($input['roles']);}
            $user->save();

            return Redirect::to_route('users')
                ->with('type', 'success')
                ->with('message', 'L\'administrateur a bien été ajouté.');
        }
    }

    public function get_edit($id)
    {
        if(!$user = User::find($id)){return Redirect::to_route('users');}

        $data['user'] = $user;
        $data['roles'] = Role::order_by('name','ASC')->get();
        $data['page_title'] = 'Modifier un utilisateur - ' . $user->email;
        return View::make('backend::users.edit', $data);
    }

    public function put_update($id)
    {
        if(!$user = User::find($id)){return Redirect::to_route('users');}

        $input = Input::all();
        $this->rules['email'] = 'required|email|unique:users,email,'.$user->id;
        unset($this->rules['password']);
        
        if(Input::get('password'))
            $this->rules['password'] = 'required|min:3';

        $v = Validator::make($input, $this->rules);

        if($v->fails())
        {
            return Redirect::to_route('edit_user', $id)
                ->with('errors', $v->errors)
                ->with_input();
        }
        else
        {
            // Update user
            $user->email 	 = $input['email'];
            $user->first_name= $input['first_name'];
            $user->last_name = $input['last_name'];
            $user->phone 	 = $input['phone'];
            
            if(Input::get('password'))
                $user->password = $input['password'];
                
            if(isset($input['roles'])){$user->roles()->sync($input['roles']);}
            
            $user_roles = array();
            foreach(Auth::user()->roles as $r){
            	if(in_array($r->id, $input['roles'])){
	            	$user_roles[] = $r->id; 
            	}
            }

            $user->save();

            return Redirect::to_route('users')
                ->with('type', 'info')
                ->with('message', 'L\'utilisateur a bien été mis à jour.');
        }
    }

    public function delete_destroy($id)
    {
        if(!$user = User::find($id)){return Redirect::to_route('users');}

        if(Auth::user()->id == $id)
            return Redirect::to_route('users')
                ->with('type', 'error')
                ->with('message', 'Auto suppression impossible !');

        $user->delete();

        return Redirect::to_route('users')
            ->with('type', 'error')
            ->with('message', 'L\'utilisateur a bien été supprimé.');
    }

    public function get_profile()
    {
        $data['page_title'] = 'Mon profil - ' . Auth::user()->first_name;
        return View::make('backend::users.profile', $data);
    }

    public function put_profile()
    {
        $input = Input::all();

        // Basic rules
        $rules['first_name'] = 'required|min:3|max:200';
        $rules['last_name'] = 'required|min:3|max:200';

        if(Input::get('new_password'))
        {
            $rules['new_password'] = 'required|min:5';
            $rules['new_password_confirmation'] = 'required|same:new_password|min:5';
        }

        $v = Validator::make($input, $rules);
        if($v->fails())
        {
            return Redirect::to_action('backend::users@profile')
                ->with('errors', $v->errors)
                ->with_input();
        }
        else
        {
            // Update user
            $user = Auth::user();
            $user->first_name = $input['first_name'];
            $user->last_name = $input['last_name'];
            if(Input::get('new_password'))
                $user->password = $input['new_password'];

            $user->save();

            return Redirect::to_action('backend::users@profile')
                ->with('success', '');
        }
    }
}