<?php

// API IOS
Route::post('api/sync', 'backend::api@sync');

// Routes for guests only
Route::group(array('before' => 'backend_logged'), function(){
	// Login
	Route::any('(:bundle)/login', 'backend::login@index');
});

// Routes needing auth
Route::group(array('before' => 'backend_auth'), function(){

	// Dashboard
	Route::get('(:bundle)', array('as' => 'home', 'uses' => 'backend::dashboard@index'));
	
	// Profile
	Route::any('(:bundle)/me', 'backend::users@profile');
	
	// Administrateurs
	Route::get('(:bundle)/users', array('as' => 'users', 'uses' => 'backend::users@index'));
	Route::get('(:bundle)/users/create', array('as' => 'new_user', 'uses' => 'backend::users@create'));
	Route::get('(:bundle)/users/(:any)/edit', array('as' => 'edit_user', 'uses' => 'backend::users@edit'));
	Route::post('(:bundle)/users', 'backend::users@store');
	Route::put('(:bundle)/users/(:any)', array('as' => 'update_user', 'uses' => 'backend::users@update'));
	Route::delete('(:bundle)/users/(:any)', array('as' => 'delete_user', 'uses' => 'backend::users@destroy'));

	// Customers
	Route::get('(:bundle)/ajaxcustomers', array('as' => 'ajaxcustomers', 'uses' => 'backend::customers@ajaxcustomers'));
	Route::get('(:bundle)/customers', array('as' => 'customers', 'uses' => 'backend::customers@index'));
	Route::get('(:bundle)/customers/create', array('as' => 'new_customer', 'uses' => 'backend::customers@create'));
	Route::get('(:bundle)/customers/(:any)/edit', array('as' => 'edit_customer', 'uses' => 'backend::customers@edit'));
	Route::post('(:bundle)/customers', 'backend::customers@store');
	Route::put('(:bundle)/customers/(:any)', array('as' => 'update_customer', 'uses' => 'backend::customers@update'));
	Route::delete('(:bundle)/customers/(:any)', array('as' => 'delete_customer', 'uses' => 'backend::customers@destroy'));
	Route::post('(:bundle)/customers/files', 'backend::customers@file');
	
	// Scans
	Route::get('(:bundle)/ajaxscans', array('as' => 'ajaxscans', 'uses' => 'backend::scans@ajaxscans'));
	Route::get('(:bundle)/scans/csv', array('as' => 'csv_scan', 'uses' => 'backend::scans@exportcsv'));
	Route::get('(:bundle)/scans', array('as' => 'scans', 'uses' => 'backend::scans@index'));
	Route::get('(:bundle)/scans/create', array('as' => 'new_scan', 'uses' => 'backend::scans@create'));
	Route::get('(:bundle)/scans/(:any)/edit', array('as' => 'edit_scan', 'uses' => 'backend::scans@edit'));
	Route::post('(:bundle)/scans', 'backend::scans@store');
	Route::put('(:bundle)/scans/(:any)', array('as' => 'update_scan', 'uses' => 'backend::scans@update'));
	Route::delete('(:bundle)/scans/(:any)', array('as' => 'delete_scan', 'uses' => 'backend::scans@destroy'));
	
	//Pdf avec QR code
	Route::get('(:bundle)/qrcodepdf/(:any)', array('as' => 'qrcodepdf', 'uses' => 'backend::customers@qrcodepdf'));

	// Logout
	Route::get('(:bundle)/logout', 'backend::login@logout');

});

/*
|--------------------------------------------------------------------------
| Route Filters
|--------------------------------------------------------------------------
|
| Filters provide a convenient method for attaching functionality to your
| routes. The built-in before and after filters are called before and
| after every request to your application, and you may even create
| other filters that can be attached to individual routes.
|
*/

Route::filter('permission_check', function($role_id)
{
	if(!Auth::user()->has_permission($role_id)){return Redirect::to_route('home');}
});

Route::filter('backend_auth', function()
{
	if (Auth::guest()) return Redirect::to_action('backend::login');
});

Route::filter('backend_logged', function()
{
	if (Auth::check()) return Redirect::to_action('backend::');
});