﻿<?php
						//----------------------
						// 	PAGE DE FONCTIONS
						//	    --------
						//	 FUNCTIONS PAGE
						//----------------------



// Cette fonction génère trans_id avec un compteur. trans_id est l'identifiant de transaction, il doit
// être unique sur une même journée, compris entre 000000 et 899999 et de longueur 6 (6 caractères).
//
// Dans cet exemple la valeur du compteur est stockée dans le fichier count.txt, incrémenté de 1 à chaque lecture 
// et remis à zéro si la valeur est superieure à 899999.
// --------
// This function generates trans_id by using a counter. trans_id is the transaction ID, it must 
// be unique within the same day, between 000000 and 899999 and have a 6 char length.
//
// In this example the counter value is written in the count.txt file, incremented of 1 on every reading
// and set back to zero if its value is higher than 899999.

function get_Trans_id() {

// ouverture et verrouillage du fichier // opening and locking of the file
	$filename = "./compteur/count.txt";// chemin du fichier // path of the file
	$fp = fopen($filename, 'r+');
	flock($fp, LOCK_EX);

// lecture et incrémentation // reading and incrementation
	$count = (int)fread($fp, 6);  
	$count++;
	if($count < 0 || $count > 899999) {
		$count = 0;
	}

// retour au début du fichier // back to the beginning of the file
	fseek($fp, 0);
	ftruncate($fp,0);

// écriture, fermeture et déverrouillage // writing, closure and unlocking
	fwrite($fp, $count);
	flock($fp, LOCK_UN);
	fclose($fp);

// rajout de zéros au début si nécessaire // adding of zeros at the beginning if necessary
	$trans_id = sprintf("%06d",$count);
	return($trans_id);
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

// Fonction récuperant la certificat dans le fichier de configuration
// --------
// Function which fetches the certificate within the configuration file

function Get_Key()
{
	$conf_txt = parse_ini_file("config/conf.txt");
	if ($conf_txt['vads_ctx_mode'] == "TEST") $key = $conf_txt['TEST_key'];
	if ($conf_txt['vads_ctx_mode'] == "PRODUCTION") $key = $conf_txt['PROD_key'];
	return $key;
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

// Fonction récuperant la mode du debug dans le fichier de configuration
// --------
// Function which fetches the debug mode within the configuration file

function Get_Debug()
{
	$conf_txt = parse_ini_file("config/conf.txt");
	if (isset($conf_txt['debug'])) return $conf_txt['debug'];
	else return 0;
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

// Calcul de la signature 
// --------
// Calculation of the signature

function get_Signature($field,$key) 
{
	ksort($field); // tri des paramétres par ordre alphabétique  // sort the settings by alphabetical order
	$contenu_signature = "";
	foreach ($field as $nom => $valeur)	{
		if(substr($nom,0,5) == 'vads_') {
		$contenu_signature .= $valeur."+";
		}
	}
	$contenu_signature .= $key;	// Ajout du certificat à la fin de la chaîne // Adding of the certificate at the end of the string
	$signature = sha1($contenu_signature);
	return($signature);
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

// Contrôle de la signature 
// --------
// Control of the signature

function Check_Signature($field,$key) {
	$result='false';
	$signature=get_Signature($field,$key);
	if(isset($field['signature']) && ($signature == $field['signature'])) $result='true';
	return ($result);
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

// Génération des champs du formulaire à envoyer à la plateforme
// Les champs doivent être encodés en UTF8 obligatoirement. Seul les champs commençant par vads_ et la signature sont envoyés.
// --------
// Generation of the form's fields that have to be sent to the gateway
// The fields must be encoded in UTF-8. Only the fields starting with vads_ and the signature are sent.

function get_formHtml_request($field) {
	
	$conf_txt = parse_ini_file("config/conf.txt"); // lecture du fichier de configuration // reading of the configuration file
	if (isset($conf_txt['TEST_key'])) unset ($conf_txt['TEST_key']);
	if (isset($conf_txt['PROD_key'])) unset ($conf_txt['PROD_key']);
	if (isset($conf_txt['debug'])) unset ($conf_txt['debug']);
	
	foreach ($conf_txt as $conf_field => $conf_value) $field[$conf_field] = $conf_value;
	
	$params=array();
	foreach ($field as $nom => $valeur){
		if(substr($nom,0,5) == 'vads_') {
				$params[$nom]=	$valeur;
		}
	}
	
	if (isset($field['vads_trans_id'])) $params['vads_trans_id'] = $field['vads_trans_id'];
	else $params['vads_trans_id'] = get_Trans_id();
	if (isset($field['vads_trans_date'])) $params['vads_trans_date'] = $field['vads_trans_date'];
	else $params['vads_trans_date'] = gmdate ("YmdHis", time());
	if (isset($field['signature'])) $params['signature'] = $field['signature'];
	else {
		$key = Get_Key();
		$params['signature'] = get_Signature($params, $key);
	}
	
	ksort($params);

	return ($params);
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

// Traitement et affichage du formulaire
// --------
// Process and display of the form

function print_formHtml_request($field, $lang = "fr") {

	include($lang.'.php');
	
	$params = get_formHtml_request($field);
	$key = Get_Key();
	$debug = Get_Debug();
	
	// Affichage d'une erreur si conf.txt n'est pas configuré // Display an error is conf.txt is not configured
	$error = false;
	if ($params['vads_site_id'] == "11111111") $error = true;
	if ($params['vads_site_id'] == "") $error = true;
	if ($key == "2222222222222222") $error = true;
	if ($key == "3333333333333333") $error = true;
	if ($key == "") $error = true;
	if (isset($params['vads_url_return']) && $params['vads_url_return'] == "") $error = true;
	if ($error == true) return ("
<div id=\"Info\">
	".$ERREUR_DE_CONFIGURATION_para."
</div>");
	
	// Debut du formulaire // Beginning of the form
	$form= '<form name="pay_form" method="POST" action="https://paiement.systempay.fr/vads-payment/">';
	
			// Affichage si debug = ON // Display if debug = ON 
			if ($debug == 1) $form.= '<div id="Info"><p><b><u>'.$POUR_INFORMATION_VOICI_LES_CHAMPS.'</u></b></p>'; 
	
	foreach ($params as $nom => $valeur) {
		$form.='<input type="hidden" name="' . $nom . '" value="' . $valeur . '" />';
		
			// Affichage des champs si debug = ON // Display the fields if debug = ON 	
			if ($debug == 1) $form.= $nom.' : '.$valeur.'<br/>'; 
	}
			// Affichage du boutton de validation si debug = ON // Display the validation button if debug = ON 
			if ($debug == 1) $form.=' 
				<button type="submit" class="validationButton" >
				<span><em>'.$PAYER.'</em></span>
				</button>'; 

	$form.='
	</div></form>'; 
	// Fin du formulaire // End of the form
	
			// Affichage si debug = ON // Display if debug = ON 
			if ($debug == 1) $form.=' 
			<div id="Info">
				'.$REMARQUE_REDIRECTION_para.'
			</div>';
	
	// Redirection directe vers la plateforme de paiement si debug = OFF // Direct redirection to the payment gateway if debug = OFF
	if ($debug != 1) $form.='
	<script language="JavaScript">
		document.pay_form.submit();
	</script>'; 

	return($form);
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

// Fonction qui crée le message destiné à la plateforme en fonction du résultat du paiement
// --------
// This function creates the message intended to the gateway depending on the payment results

function getOutputForGateway($case = '', $lang, $encoding = "UTF-8") 
{
	include($lang.'.php');

	// Messages prédéfinis selon les cas // Predefined messages depending on the cases
	$message = '';
	switch ($case) {
		case "signature_invalide":
			$message = $signature_invalide;
		break;
		case "AUTHORISED":
			$message = "Signature OK ".$AUTHORISED;
		break;
		case "REFUSED":
			$message = "Signature OK ".$REFUSED;
		break;
		case "AUTHORISED_TO_VALIDATE":
			$message = "Signature OK ".$AUTHORISED_TO_VALIDATE;
		break;
		case "WAITING_AUTHORISATION":
			$message = "Signature OK ".$WAITING_AUTHORISATION;
		break;
		case "WAITING_AUTHORISATION_TO_VALIDATE":
			$message = "Signature OK ".$WAITING_AUTHORISATION_TO_VALIDATE;
		break;
	}

	// Gestion de l'encodage, si le message n'est pas encodé en UTF-8 on l'encode en UTF-8
	// --------
	// Encoding management, if it is not UTF-8 we code it into UTF-8
	if($encoding !== "UTF-8") {
		$message = iconv($encoding, "UTF-8", $message);
	}

	// Permet d'afficher un span caché avec le résultat du paiement.
	// Cette réponse sera traitée par la plateforme de paiement et affichée dans le back office de la solution.
	// --------
	// Displays a hidden span which contains the paylent result
	// This answer will be treated by the gateway and displayed in the solution back office
	$response = ' <span style="display:none">';
	$response .= $message;
	$response .= '</span>';
	return $response;
}

?>